package envoy.client.ui;

import java.time.format.DateTimeFormatter;
import java.util.Map;

import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.VBox;

import envoy.data.Message;
import envoy.data.Message.MessageStatus;
import envoy.data.User;

/**
 * Displays a single message inside the message list.
 * <p>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>MessageListCell.java</strong><br>
 * Created: <strong>28.03.2020</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Client v0.1-beta
 */
public class MessageListCell extends ListCell<Message> {

	private final ListView<Message> listView;

	private static User								client;
	private static final DateTimeFormatter			dateFormat		= DateTimeFormatter.ofPattern("dd.MM.yyyy HH:mm");
	private static final Map<MessageStatus, Image>	statusImages	= IconUtil.loadByEnum(MessageStatus.class, 16);

	/**
	 * @param listView the list view inside which this cell is contained
	 * @since Envoy Client v0.1-beta
	 */
	public MessageListCell(ListView<Message> listView) { this.listView = listView; }

	/**
	 * Displays the text, the data of creation and the status of a message.
	 *
	 * @since Envoy v0.1-beta
	 */
	@Override
	protected void updateItem(Message message, boolean empty) {
		super.updateItem(message, empty);
		if (empty || message == null) {
			setText(null);
			setGraphic(null);
		} else {
			final var	cell		= new VBox(new Label(dateFormat.format(message.getCreationDate())));
			final var	textLabel	= new Label(message.getText());
			textLabel.setWrapText(true);
			cell.getChildren().add(textLabel);
			if (message.getRecipientID() != client.getID()) {
				cell.getChildren().add(new Label("", new ImageView(statusImages.get(message.getStatus()))));
				cell.getStyleClass().add("own-message");
			} else cell.getStyleClass().add("received-message");
			cell.paddingProperty().setValue(new Insets(5, 20, 5, 20));
			cell.prefWidthProperty().bind(listView.widthProperty().subtract(40));
			setGraphic(cell);
		}
	}

	/**
	 * @param client the user who chats with another person
	 * @since Envoy Client v0.1-beta
	 */
	public static void setUser(User client) { MessageListCell.client = client; }
}
